--- src/bsd_mouse.c.orig	2012-10-08 10:40:07.000000000 +0900
+++ src/bsd_mouse.c	2013-06-16 23:20:13.000000000 +0900
@@ -33,6 +33,7 @@
 #include "xf86Priv.h"
 #include "xf86_OSlib.h"
 #include "xf86Xinput.h"
+#include "xorgVersion.h"
 #include "mouse.h"
 #include "xisb.h"
 #include "mipointer.h"
@@ -53,12 +54,15 @@
 
 #define HUP_GENERIC_DESKTOP     0x0001
 #define HUP_BUTTON              0x0009
+#define HUP_CONSUMER            0x000c
 
 #define HUG_X                   0x0030
 #define HUG_Y                   0x0031
 #define HUG_Z                   0x0032
 #define HUG_WHEEL               0x0038
 
+#define HUC_AC_PAN              0x0238
+
 #define HID_USAGE2(p,u) (((p) << 16) | u)
 
 /* The UMS mices have middle button as number 3 */
@@ -98,7 +102,7 @@
 SupportedInterfaces(void)
 {
 #if defined(__NetBSD__)
-    return MSE_SERIAL | MSE_BUS | MSE_PS2 | MSE_AUTO;
+    return MSE_SERIAL | MSE_BUS | MSE_PS2 | MSE_AUTO | MSE_MISC;
 #elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || defined(__DragonFly__)
     return MSE_SERIAL | MSE_BUS | MSE_PS2 | MSE_AUTO | MSE_MISC;
 #else
@@ -339,6 +343,23 @@
 }
 #endif
 
+#if defined(__NetBSD__)
+static Bool
+SetupMouse(InputInfoPtr pInfo)
+{
+#ifdef WSCONS_SUPPORT
+#ifdef WSMOUSEIO_SETVERSION
+    int version = WSMOUSE_EVENT_VERSION;
+    if (ioctl(pInfo->fd, WSMOUSEIO_SETVERSION, &version) == -1) {
+        xf86Msg(X_WARNING, "%s: cannot set version\n", pInfo->name);
+        return FALSE;
+    }
+#endif
+#endif
+    return TRUE;
+}
+#endif
+
 #if (defined(__OpenBSD__) || defined(__NetBSD__)) && defined(WSCONS_SUPPORT)
 
 /* Only support wsmouse configuration for now */
@@ -409,7 +430,7 @@
     n /= sizeof(struct wscons_event);
     while( n-- ) {
         int buttons = pMse->lastButtons;
-        int dx = 0, dy = 0, dz = 0, dw = 0;
+        int dx = 0, dy = 0, dz = 0, dw = 0, x, y;
         switch (event->type) {
         case WSCONS_EVENT_MOUSE_UP:
 #define BUTBIT (1 << (event->value <= 2 ? 2 - event->value : event->value))
@@ -434,6 +455,38 @@
             dw = event->value;
             break;
 #endif
+	case WSCONS_EVENT_MOUSE_ABSOLUTE_X:
+	    miPointerGetPosition (pInfo->dev, &x, &y);
+	    x = event->value;
+#if XORG_VERSION_MINOR >= 11
+	    miPointerSetPosition (pInfo->dev, Absolute, (double *)&x, (double *)&y);
+#else
+	    miPointerSetPosition (pInfo->dev, &x, &y);
+#endif
+	    xf86PostMotionEvent(pInfo->dev, TRUE, 0, 2, x, y);
+	    ++event;
+	    continue;
+	case WSCONS_EVENT_MOUSE_ABSOLUTE_Y:
+	    miPointerGetPosition (pInfo->dev, &x, &y);
+	    y = event->value;
+#if XORG_VERSION_MINOR >= 11
+	    miPointerSetPosition (pInfo->dev, Absolute, (double *)&x, (double *)&y);
+#else
+	    miPointerSetPosition (pInfo->dev, &x, &y);
+#endif
+	    xf86PostMotionEvent(pInfo->dev, TRUE, 0, 2, x, y);
+	    ++event;
+	    continue;
+#ifdef WSCONS_EVENT_MOUSE_ABSOLUTE_Z
+	case WSCONS_EVENT_MOUSE_ABSOLUTE_Z:
+	    ++event;
+	    continue;
+#endif
+#ifdef WSCONS_EVENT_MOUSE_ABSOLUTE_W
+	case WSCONS_EVENT_MOUSE_ABSOLUTE_W:
+	    ++event;
+	    continue;
+#endif
         default:
             LogMessageVerbSigSafe(X_WARNING, -1,
                                   "%s: bad wsmouse event type=%d\n", pInfo->name,
@@ -474,7 +527,7 @@
     hid_item_t loc_x;           /* x locator item */
     hid_item_t loc_y;           /* y locator item */
     hid_item_t loc_z;           /* z (wheel) locator item */
-    hid_item_t loc_w;           /* z (wheel) locator item */
+    hid_item_t loc_w;		/* w (pan) locator item */
     hid_item_t loc_btn[MSE_MAXBUTTONS]; /* buttons locator items */
    unsigned char *buffer;
 } UsbMseRec, *UsbMsePtr;
@@ -690,6 +743,9 @@
     if (hid_locate(reportDesc, HID_USAGE2(HUP_GENERIC_DESKTOP, HUG_WHEEL),
                    hid_input, &pUsbMse->loc_z, pUsbMse->iid) < 0) {
     }
+    if (hid_locate(reportDesc, HID_USAGE2(HUP_CONSUMER, HUC_AC_PAN),
+                   hid_input, &pUsbMse->loc_w, pUsbMse->iid) < 0) {
+    }
 #else
     if (hid_locate(reportDesc, HID_USAGE2(HUP_GENERIC_DESKTOP, HUG_X),
                    hid_input, &pUsbMse->loc_x) < 0) {
@@ -702,6 +758,9 @@
     if (hid_locate(reportDesc, HID_USAGE2(HUP_GENERIC_DESKTOP, HUG_WHEEL),
                    hid_input, &pUsbMse->loc_z) < 0) {
     }
+    if (hid_locate(reportDesc, HID_USAGE2(HUP_CONSUMER, HUC_AC_PAN),
+                   hid_input, &pUsbMse->loc_w) < 0) {
+    }
 #endif
     /* Probe for number of buttons */
     for (i = 1; i <= MSE_MAXBUTTONS; i++) {
@@ -770,6 +829,9 @@
     p->SetBMRes = SetSysMouseRes;
     p->SetMiscRes = SetSysMouseRes;
 #endif
+#if defined(__NetBSD__)
+    p->SetupMouse = SetupMouse;
+#endif
 #if (defined(__OpenBSD__) || defined(__NetBSD__)) && defined(WSCONS_SUPPORT)
     p->SetupAuto = SetupAuto;
     p->SetMiscRes = SetMouseRes;
